package com.okseby.engine;

public class GameContainer implements Runnable {
    private Thread thread;
    private Window window;
    private Renderer renderer;
    public Input input;
    private AbstractGame game;

    private boolean running = false;
    private final double TPS = 1.0 / 60.0;

    private String metrics;

    public GameContainer(AbstractGame game) {
        this.game = game;
    }

    public void start() {
        window = new Window();
        window.create();
        
        renderer = new Renderer();
        input = new Input();
        
        thread = new Thread(this);
        thread.run();
    }

    public void stop() {

    }

    public void run() {
        running = true;

        boolean render = false;

        double firstTime = 0;
        double lastTime = System.nanoTime() / 1000000000.0;
        double passedTime = 0;
        double unprocessedTime = 0;

        double tickTime = 0;
        int ticks = 0;
        int tps = 0;
        
        while (running) {
            render = false;

            firstTime = System.nanoTime() / 1000000000.0;
            passedTime = firstTime - lastTime;
            lastTime = firstTime;

            unprocessedTime += passedTime;
            tickTime += passedTime;

            while (unprocessedTime >= TPS) {
                unprocessedTime -= TPS;
                render = true;

                //TODO: Gameloop
                game.update((float)TPS);

                metrics = "TPS: " + tps + " | " + "Mouse X: " + input.mouseX + " | " + "Mouse Y: " + input.mouseY + " | " + "Scroll: " + input.scroll;
                Window.window.setTitle(Window.title + " | " + metrics);

                input.update();

                if (tickTime >= 1.0) {
                    tickTime = 0;
                    tps = ticks;
                    ticks = 0;
                }
            }

            if (render) {
                //renderer.test();
                renderer.clear();
                game.render(renderer);
                
                Window.update();
                ticks++;
            } else {
                try {
                    Thread.sleep(1);
                } catch (InterruptedException e) {
                    e.printStackTrace();   
                }
            }
        }

        dispose();
    }

    public void dispose() {
        
    }
}